# -*- coding: utf-8 -*-
from datetime import datetime
import traceback
from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import get_srid, convert_srid, format_f, parse_geom

try:
    from cStringIO import StringIO
except:
    from StringIO import StringIO


class F9(StopMultiprocessBased):
    def __init__(self, **kwargs):
        super(F9, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'F9 - Topologia real de la xarxa '
        self.base_object = 'Trams AT i BT'
        o = self.connection
        wiz_obj = o.WizardCircular4_2015
        self.codi_r1 = wiz_obj.default_get(['codi_r1'])['codi_r1']
        id_res_like = o.ResConfig.search(
            [('name', '=', 'giscegis_btlike_layer')])
        self.layer = 'LBT\_%'
        if id_res_like:
            self.layer = o.ResConfig.read(id_res_like, ['value'])[0]['value']
        self.alternative = kwargs.get('alternative', False)

    def get_sequence(self):
        """
        Generates the sequence of ids to be passed to the consummer function

        :return: ids to process
        :rtype: list[int]
        """

        o = self.connection
        data_pm_limit = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        static_search_params = ['|',
                                ('data_pm', '=', False),
                                ('data_pm', '<', data_pm_limit),
                                '|',
                                ('data_baixa', '>', data_baixa),
                                ('data_baixa', '=', False),
                                ]
        # Revisem que si està de baixa ha de tenir la data informada.
        static_search_params += ['|',
                                 '&',
                                 ('active', '=', False),
                                 ('data_baixa', '!=', False),
                                 ('active', '=', True)
                                 ]
        # AT
        trams = []
        ids_linia_at = o.GiscedataAtLinia.search([])
        fict_line_id = o.GiscedataAtLinia.search(
            [('name', '=', '1')], 0, 0, False, {'active_test': False})
        ids_linia_at += fict_line_id
        linies = o.GiscedataAtLinia.read(ids_linia_at, ['trams'])
        for elem in linies:
            trams += elem['trams']
        search_params = [('id', 'in', trams),
                         ('cini', '!=', '0000000')]
        search_params += static_search_params
        ids_at = o.GiscedataAtTram.search(
            search_params, 0, 0, False, {'active_test': False})
        search_params = [('id', 'in', trams),
                         ('cini', '!=', '0000000'),
                         ('longitud_cad', '>', 100),
                         ('cable.tipus.codi', '=', 'E')]
        search_params += static_search_params
        remove_ids_at = o.GiscedataAtTram.search(
            search_params, 0, 0, False, {'active_test': False})
        ids_at = list(set(ids_at)-set(remove_ids_at))
        # BT

        search_params = [
            ('cable.tipus.codi', 'in', ['T', 'D', 'S', 'E', 'I'])
        ]
        search_params += static_search_params
        ids_bt = o.GiscedataBtElement.search(
            search_params, 0, 0, False, {'active_test': False})
        # IDS AT + BT

        ids = []
        for at in ids_at:
            ids.append((at, 'at'))
        for bt in ids_bt:
            ids.append((bt, 'bt'))

        return ids

    def get_geom(self, id_tram, net):
        """
        Returns the geometry of the first point of the tram

        :param id_tram: Identifier of the tram
        :type id_tram: int
        :param net: type of network (AT,BT)
        :type net: str
        :return: X, Y
        :rtype: tuple[float,float]
        """

        o = self.connection
        like_layer = self.layer
        model_edge = o.GiscegisEdge
        model_vertex = o.GiscegisPolylineVertex
        model_polyline = o.GiscegisPolyline
        data = []
        if net.lower() == 'at':
            ids_edges = model_edge.search(
                [('id_linktemplate', '=', id_tram),
                 ('layer', 'not ilike', like_layer),
                 ('layer', 'not ilike', 'EMBARRA%BT%')]
            )
        else:
            ids_edges = model_edge.search(
                [('id_linktemplate', '=', id_tram),
                 '|',
                 ('layer', 'ilike', like_layer),
                 ('layer', 'ilike', 'EMBARRA%BT%')]
            )
        edges = model_edge.read(ids_edges)
        if not edges:
            return []
        vertexs = model_polyline.read(edges[0]['polyline'][0])
        for punt in model_vertex.read(vertexs['vertex_ids']):
            data.append({'x': punt['x'],
                         'y': punt['y']})
        return data

    def conv_text(self, data):
        """
        Converts the projection of a data

        :param data: list of elemnets to convert
        :return: Elements with the projection converted
        """
        o = self.connection
        t = ''
        for line in data:
            res_srid = convert_srid(get_srid(o), [line['x'], line['y']])
            t += '{0};{1};{2}\n'.format(
                format_f(res_srid[0], decimals=6),
                format_f(res_srid[1], decimals=6),
                '')
        return t[:-1]

    def conv_text_alt(self, data):
        """
        Converts the projection of a data

        :param data: list of elemnets to convert
        :return: Elements with the projection converted
        """
        o = self.connection
        t = ''
        for line in data:
            res_srid = convert_srid(get_srid(o), [line['x'], line['y']])
            t += 'R1-{0};t_name;{1};{2};{3};1\n'.format(
                self.codi_r1,
                format_f(res_srid[0], decimals=6),
                format_f(res_srid[1], decimals=6),
                ''
            )
        return t[:-1]

    def consumer(self):
        """
        Function that generates each line of the file

        :return: None
        :rtype: None
        """
        o = self.connection
        while True:
            try:
                # generar linies
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                if item[1] == 'at':
                    if 'geom' in o.GiscedataAtTram.fields_get().keys():
                        at = o.GiscedataAtTram.read(item[0], ['geom', 'name'])
                        if at['geom']:
                            data = parse_geom(at['geom'])
                        else:
                            data = self.get_geom(at['name'], 'at')
                    else:
                        at = o.GiscedataAtTram.read(item[0], ['name'])
                        data = self.get_geom(at['name'], 'at')
                    if self.alternative:
                        data = self.conv_text_alt(data)
                        linia = data.replace('t_name', 'A' + str(at['name']))
                    else:
                        data = self.conv_text(data)
                        linia = 'A{0}\n{1}\nEND'.format(at['name'], data)
                else:
                    if 'geom' in o.GiscedataBtElement.fields_get().keys():
                        bt = o.GiscedataBtElement.read(
                            item[0], ['geom', 'name']
                        )
                        if bt['geom']:
                            data = parse_geom(bt['geom'])
                        else:
                            data = self.get_geom(bt['name'], 'bt')
                    else:
                        bt = o.GiscedataBtElement.read(item[0], ['name'])
                        data = self.get_geom(bt['name'], 'bt')
                    if self.alternative:
                        data = self.conv_text_alt(data)
                        linia = data.replace('t_name', 'B' + str(bt['name']))
                    else:
                        data = self.conv_text(data)
                        linia = 'B{0}\n{1}\nEND'.format(bt['name'], data)
                self.output_q.put([linia])
                self.input_q.task_done()
            except:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()

    def writer(self):
        if self.file_output:
            fio = open(self.file_output, 'wb')
        else:
            fio = StringIO()
        while True:
            try:
                item = self.output_q.get()
                if item == 'STOP':
                    break
                msg = map(
                    lambda x: type(x) == unicode and x.encode('utf-8') or x,
                    item
                )
                fio.write(str(msg[0])+'\n')
            except Exception:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.output_q.task_done()
        fio.write('END')
        if not self.file_output:
            self.content = fio.getvalue()
        fio.close()

        if not self.file_output:
            self.content = fio.getvalue()
        fio.close()
        if self.file_modificaciones:
            fio_mod = open(self.file_modificaciones, 'wb')
        else:
            fio_mod = StringIO()

        while True:
            try:
                val = self.output_m.get()
                if val == 'STOP':
                    break
                fio_mod.writelines(val + "\n")
                self.output_m.task_done()
            except:
                self.output_m.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
        fio_mod.close()



from LAT import LAT
from LBT import LBT
from SUB import SUB
from POS import POS, POS_INT
from MAQ import MAQ
from DES import DES
from FIA import FIA
from CTS import CTS
from CON import CON
from MOD import ModCts, ModFia, ModLat, ModLbt, ModMaq, ModPos



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC Subestacions
"""
from __future__ import absolute_import
from datetime import datetime
import traceback
import sys

from libcnmc.core import MultiprocessBased
from libcnmc.utils import get_id_municipi_from_company, format_f
from libcnmc.models import F3Res4771, F3Res4131

QUIET = False


class SUB_2015(MultiprocessBased):
    """
    Class that generates the SUB(3) report of the 4131
    """
    def __init__(self, **kwargs):
        """
        Class constructor
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: CT
        """
        super(SUB_2015, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'Subestacions'
        self.report_name = 'CNMC INVENTARI SUB'
        self.compare_field = kwargs["compare_field"]

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consummer
        :return: List of ids
        """
        search_params = []
        data_pm = '{}-01-01'.format(self.year + 1)
        data_baixa = '{}-01-01'.format(self.year)
        search_params += [('propietari', '=', True),
                          '|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                               ('data_baixa', '=', False)
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataCtsSubestacions.search(
            search_params, 0, 0, False, {'active_test': False})

    def consumer(self):
        """
        Method that generates the csb file
        :return: List of arrays
        """
        O = self.connection
        fields_to_read = [
            'name', 'data_industria', 'data_pm', 'id_municipi', 'cini',
            'descripcio', 'perc_financament', 'data_baixa', 'posicions',
            'cnmc_tipo_instalacion', self.compare_field
        ]
        data_pm_limit = '{}-01-01'.format(self.year + 1)
        data_baixa_limit = '{}-01-01'.format(self.year)
        error_msg = "**** ERROR: El ct %s (id:%s) no està en giscedata_cts_subestacions.\n"
        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)

                sub = O.GiscedataCtsSubestacions.read(item, fields_to_read)

                if not sub:
                    txt = (error_msg.format((sub['name'], sub['id'])))
                    if not QUIET:
                        sys.stderr.write(txt)
                        sys.stderr.flush()

                    raise Exception(txt)

                # Calculem any posada en marxa
                data_pm = sub['data_pm']

                if data_pm:
                    data_pm = datetime.strptime(str(data_pm), '%Y-%m-%d')
                    data_pm = data_pm.strftime('%d/%m/%Y')

                comunitat = ''
                if sub['id_municipi']:
                    id_municipi = sub['id_municipi'][0]
                else:
                    id_municipi = get_id_municipi_from_company(O)

                if id_municipi:
                    #funció per trobar la ccaa desde el municipi
                    fun_ccaa = O.ResComunitat_autonoma.get_ccaa_from_municipi
                    id_comunitat = fun_ccaa(id_municipi)
                    comunitat_vals = O.ResComunitat_autonoma.read(
                        id_comunitat[0], ['codi'])
                    if comunitat_vals:
                        comunitat = comunitat_vals['codi']
                if sub['data_baixa']:
                    if sub['data_baixa'] < data_pm_limit:
                        fecha_baja = sub['data_baixa']
                    else:
                        fecha_baja = ''
                else:
                    fecha_baja = ''

                if 'posicions' in sub:
                    num_pos = 0
                    for pos in sub['posicions']:
                        pos_data = O.GiscedataCtsSubestacionsPosicio.read(pos, ['interruptor'])
                        if pos_data['interruptor'] == '2':
                            num_pos += 1
                else:
                    num_pos = 1

                if sub[self.compare_field]:
                    last_data = sub[self.compare_field]
                    entregada = F3Res4131(**last_data)
                    actual = F3Res4131(
                        sub['name'],
                        sub['cini'],
                        sub['descripcio'],
                        comunitat,
                        format_f(round(100 - int(sub['perc_financament']))),
                        data_pm,
                        num_pos
                    )
                    if entregada == actual:
                        estado = 0
                    else:
                        estado = 1
                else:
                    estado = 2
                    
                output = [
                    '{0}'.format(sub['name']),
                    sub['cini'] or '',
                    sub['descripcio'] or '',
                    comunitat,
                    format_f(round(100 - int(sub['perc_financament']))),
                    data_pm,
                    fecha_baja,
                    num_pos,
                    estado
                ]

                self.output_q.put(output)
            except Exception:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()


class SUB(MultiprocessBased):
    """
    Class that generates the SUB(3) report of the 4131
    """

    def __init__(self, **kwargs):
        """
        Class constructor
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: CT
        """
        super(SUB, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'Subestacions'
        self.report_name = 'CNMC INVENTARI SUB'
        self.compare_field = kwargs["compare_field"]

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consummer
        :return: List of ids
        """
        search_params = []
        data_pm = '{}-01-01'.format(self.year + 1)
        data_baixa = '{}-01-01'.format(self.year)
        search_params += [('propietari', '=', True),
                          '|', ('data_pm', '=', False),
                          ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                          ('data_baixa', '=', False)
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                          ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataCtsSubestacions.search(
            search_params, 0, 0, False, {'active_test': False})

    def consumer(self):
        """
        Method that generates the csb file
        :return: List of arrays
        """
        O = self.connection
        fields_to_read = [
            'name', 'data_industria', 'data_pm', 'id_municipi', 'cini',
            'descripcio', 'perc_financament', 'data_baixa', 'posicions',
            'cnmc_tipo_instalacion', self.compare_field
        ]
        data_pm_limit = '{}-01-01'.format(self.year + 1)
        data_baixa_limit = '{}-01-01'.format(self.year)
        error_msg = "**** ERROR: El ct %s (id:%s) no està en giscedata_cts_subestacions.\n"
        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)

                sub = O.GiscedataCtsSubestacions.read(item, fields_to_read)

                if not sub:
                    txt = (error_msg.format((sub['name'], sub['id'])))
                    if not QUIET:
                        sys.stderr.write(txt)
                        sys.stderr.flush()

                    raise Exception(txt)

                # Calculem any posada en marxa
                data_pm = sub['data_pm']

                if data_pm:
                    data_pm = datetime.strptime(str(data_pm), '%Y-%m-%d')
                    data_pm = data_pm.strftime('%d/%m/%Y')

                comunitat = ''
                if sub['id_municipi']:
                    id_municipi = sub['id_municipi'][0]
                else:
                    id_municipi = get_id_municipi_from_company(O)

                if id_municipi:
                    # funció per trobar la ccaa desde el municipi
                    fun_ccaa = O.ResComunitat_autonoma.get_ccaa_from_municipi
                    id_comunitat = fun_ccaa(id_municipi)
                    comunitat_vals = O.ResComunitat_autonoma.read(
                        id_comunitat[0], ['codi'])
                    if comunitat_vals:
                        comunitat = comunitat_vals['codi']
                if sub['data_baixa']:
                    if sub['data_baixa'] < data_pm_limit:
                        fecha_baja = sub['data_baixa']
                    else:
                        fecha_baja = ''
                else:
                    fecha_baja = ''

                if 'posicions' in sub:
                    num_pos = 0
                    for pos in sub['posicions']:
                        pos_data = O.GiscedataCtsSubestacionsPosicio.read(pos, [
                            'interruptor'])
                        if pos_data['interruptor'] == '2':
                            num_pos += 1
                else:
                    num_pos = 1

                if sub[self.compare_field]:
                    last_data = sub[self.compare_field]
                    entregada = F3Res4131(**last_data)
                    actual = F3Res4131(
                        sub['name'],
                        sub['cini'],
                        sub['descripcio'],
                        comunitat,
                        format_f(round(100 - int(sub['perc_financament']))),
                        data_pm,
                        fecha_baja,
                        num_pos,
                        0
                    )
                    if entregada == actual:
                        estado = 0
                    else:
                        estado = 1
                else:
                    estado = 2

                output = [
                    '{0}'.format(sub['name']),
                    sub['cini'] or '',
                    sub['descripcio'] or '',
                    comunitat,
                    format_f(round(100 - int(sub['perc_financament'])), 3),
                    data_pm,
                    fecha_baja,
                    num_pos,
                    estado
                ]

                self.output_q.put(output)
            except Exception:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()




def get_resum_any_id(connection, any):
    """
    Returnt the id of the resum from the year
     
    :param connection: OOOP Connection 
    :param any: Year
    :type any: int, str 
    :return: id of the resum
    :rtype: int
    """

    search_res = [("anyo", "=", any)]
    ids = connection.GiscedataCnmcPla_inversio.search(search_res)

    search_pla = [("pla_inversio", "=", ids[0])]
    id_resum = connection.GiscedataCnmcResum_any.search(search_pla)
    return id_resum



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import traceback
from os import environ

from libcnmc.core import MultiprocessBased


class SE(MultiprocessBased):
    """
    Class to generate installation report for SE (3)
    """

    def __init__(self, **kwargs):
        """
        Class constructor
        :param kwargs: 
        """

        self.year = kwargs.pop("year")

        super(SE, self).__init__(**kwargs)
        self.include_obres = False
        if kwargs.get("include_obra", False):
            self.include_obres = True
        if kwargs.get("include_header", False):
            self.file_header = self.get_header()

    def get_header(self):
        header = [
            'IDENTIFICADOR',
            'CINI',
            'DENOMINACION',
            'TIPO_SUELO',
            'PLANIFICACION',
            'IDENTIFICADOR_PARQUE_1',
            'IDENTIFICADOR_PARQUE_2',
            'IDENTIFICADOR_PARQUE_3',
            'IDENTIFICADOR_PARQUE_4',
            'CINI_PARQUE_1',
            'CINI_PARQUE_2',
            'CINI_PARQUE_3',
            'CINI_PARQUE_4',
            'NUM_POSICIONES_PARQUE_1',
            'NUM_POSICIONES_PARQUE_2',
            'NUM_POSICIONES_PARQUE_3',
            'NUM_POSICIONES_PARQUE_4',
            'PN_TRANSFORMACION',
            'PN_REACTANCIAS',
            'PN_CONDENSADORES',
        ]
        if self.include_obres:
            header.insert(0, 'IDENTIFICADOR_OBRA')
        return header

    def get_sequence(self):
        """
        Generates the sequence of ids to make the report

        :return: List of ids
        :rtype: list
        """

        installations_ids = self.connection.GiscedataProjecteObra.get_audit_installations_by_year(
            [], self.year, [3]
        )

        return installations_ids[3]

    def consumer(self):
        """
        Generates the line of the file
        :return: Line
        :rtype: str
        """

        O = self.connection
        fields_to_read = [
            'name',
            'cini',
            'denominacion',
            'tipo_suelo',
            'planificacion',
            'identificador_parque_1',
            'identificador_parque_2',
            'identificador_parque_3',
            'identificador_parque_4',
            'cini_parque_1',
            'cini_parque_2',
            'cini_parque_3',
            'cini_parque_4',
            'num_posiciones_parque_1',
            'num_posiciones_parque_2',
            'num_posiciones_parque_3',
            'num_posiciones_parque_4',
            'pn_transformacion',
            'pn_reactancias',
            'pn_condensadores',
            'obra_id'
        ]

        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)

                linia = O.GiscedataProjecteObraTiSubestacions.read(
                    [item], fields_to_read
                )[0]
                output = [
                    linia['name'],
                    linia['cini'],
                    linia['denominacion'],
                    linia['tipo_suelo'],
                    linia['planificacion'],
                    linia['identificador_parque_1'],
                    linia['identificador_parque_2'],
                    linia['identificador_parque_3'],
                    linia['identificador_parque_4'],
                    linia['cini_parque_1'],
                    linia['cini_parque_2'],
                    linia['cini_parque_3'],
                    linia['cini_parque_4'],
                    linia['num_posiciones_parque_1'],
                    linia['num_posiciones_parque_2'],
                    linia['num_posiciones_parque_3'],
                    linia['num_posiciones_parque_4'],
                    linia['pn_transformacion'],
                    linia['pn_reactancias'],
                    linia['pn_condensadores'],
                ]
                if self.include_obres:
                    output.insert(0, linia['obra_id'][1])
                output = map(lambda e: '' if e is False or e is None else e, output)
                self.output_q.put(output)

            except Exception:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



from f1_4666 import F1Res4666
from f2_4666 import F2Res4666
from f3_4666 import F3Res4666
from f4_4666 import F4Res4666
from f5_4666 import F5Res4666
from f6_4666 import F6Res4666
from f7_4666 import F7Res4666
from f8_4666 import F8Res4666



# -*- coding: utf-8 -*-
from datetime import datetime
from multiprocessing import Manager
import re
import traceback

from libcnmc.utils import CODIS_TARIFA, CODIS_ZONA, CINI_TG_REGEXP
from libcnmc.utils import get_ine, get_comptador
from libcnmc.core import MultiprocessBased


class F1(MultiprocessBased):
    def __init__(self, **kwargs):
        super(F1, self).__init__(**kwargs)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.year = kwargs.pop('year', datetime.now().year - 1)
        manager = Manager()
        self.cts = manager.dict()
        self.base_object = 'CUPS'
        self.report_name = 'F1 - CUPS'

    def get_codi_tarifa(self, codi_tarifa):
        return CODIS_TARIFA.get(codi_tarifa, '')

    def get_sequence(self):
        search_params = []
        return self.connection.GiscedataCupsPs.search(search_params)

    def get_zona_qualitat(self, codi_ct):
        zona_qualitat = ''
        if codi_ct:
            if codi_ct in self.cts:
                return self.cts[codi_ct]
            else:
                ct_ids = self.connection.GiscedataCts.search(
                    [('name', '=', codi_ct)])
                if ct_ids:
                    dades_ct = self.connection.GiscedataCts.read(
                        ct_ids[0], ['zona_id'])
                    zona_desc = dades_ct['zona_id'][1].upper().replace(' ', '')
                    if zona_desc in CODIS_ZONA:
                        zona_qualitat = CODIS_ZONA[zona_desc]
                        self.cts[codi_ct] = zona_qualitat
        return zona_qualitat

    def get_tipus_connexio(o, id_escomesa):
        bloc = o.GiscegisBlocsEscomeses.search(
            [('escomesa', '=', id_escomesa)]
        )
        tipus = ''
        if bloc:
            bloc = o.GiscegisBlocsEscomeses.read(bloc[0], ['node'])
            if bloc['node']:
                node = bloc['node'][0]
                edge = o.GiscegisEdge.search(
                    ['|', ('start_node', '=', node), ('end_node', '=', node)]
                )
                if edge:
                    edge = o.GiscegisEdge.read(edge[0], ['id_linktemplate'])
                    if edge['id_linktemplate']:
                        bt = o.GiscedataBtElement.search(
                            [('id', '=', edge['id_linktemplate'])]
                        )
                        if bt:
                            bt = o.GiscedataBtElement.read(
                                bt[0], ['tipus_linia']
                            )
                            if bt['tipus_linia']:
                                if bt['tipus_linia'][0] == 1:
                                    tipus = 'A'
                                else:
                                    tipus = 'S'
        return tipus

    def consumer(self):
        o_codi_r1 = 'R1-%s' % self.codi_r1[-3:]
        O = self.connection
        ultim_dia_any = '%s-12-31' % self.year
        search_glob = [
            ('state', 'not in', ('esborrany', 'validar')),
            ('data_alta', '<=', ultim_dia_any),
            '|',
            ('data_baixa', '>=', ultim_dia_any),
            ('data_baixa', '=', False)
        ]
        context_glob = {'date': ultim_dia_any, 'active_test': False}
        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)
                fields_to_read = [
                    'name', 'id_escomesa', 'id_municipi', 'cne_anual_activa',
                    'cne_anual_reactiva', 'cnmc_potencia_facturada', 'et'
                ]

                cups = O.GiscedataCupsPs.read(item, fields_to_read)
                if not cups or not cups.get('name'):
                    self.input_q.task_done()
                    continue
                o_name = cups['name'][:22]
                o_codi_ine = ''
                o_codi_prov = ''
                o_zona = ''
                o_potencia_facturada = cups['cnmc_potencia_facturada'] or ''
                if 'et' in cups:
                    o_zona = self.get_zona_qualitat(cups['et'])
                if cups['id_municipi']:
                    municipi = O.ResMunicipi.read(
                        cups['id_municipi'][0], ['ine']
                    )
                    ine = get_ine(self.connection, municipi['ine'])
                    o_codi_ine = ine[1]
                    o_codi_prov = ine[0]

                o_utmx = ''
                o_utmy = ''
                o_utmz = ''
                o_nom_node = ''
                o_tensio = ''
                o_connexio = ''
                if cups and cups['id_escomesa']:
                    o_connexio = self.get_tipus_connexio(O,
                        cups['id_escomesa'][0]
                    )
                    search_params = [('escomesa', '=', cups['id_escomesa'][0])]
                    bloc_escomesa_id = O.GiscegisBlocsEscomeses.search(
                        search_params
                    )
                    if bloc_escomesa_id:
                        bloc_escomesa = O.GiscegisBlocsEscomeses.read(
                            bloc_escomesa_id[0], ['node', 'vertex']
                        )
                        if bloc_escomesa['vertex']:
                            vertex = O.GiscegisVertex.read(
                                bloc_escomesa['vertex'][0], ['x', 'y']
                            )
                            o_utmx = round(vertex['x'], 3)
                            o_utmy = round(vertex['y'], 3)
                        if bloc_escomesa['node']:
                            search_params = [('start_node', '=',
                                              bloc_escomesa['node'][0])]
                            edge_id = O.GiscegisEdge.search(search_params)
                            if not edge_id:
                                search_params = [('end_node', '=',
                                                  bloc_escomesa['node'][0])]
                                edge_id = O.GiscegisEdge.search(search_params)
                            if edge_id:
                                edge = O.GiscegisEdge.read(
                                    edge_id[0], ['name']
                                )
                                o_nom_node = edge['name']

                search_params = [('cups', '=', cups['id'])] + search_glob
                polissa_id = O.GiscedataPolissa.search(
                    search_params, 0, 1, 'data_alta desc', context_glob)
                o_potencia = ''
                o_cnae = ''
                o_pot_ads = ''
                o_equip = 'MEC'
                o_cod_tfa = ''
                o_estat_contracte = 0
                if polissa_id:
                    fields_to_read = [
                        'potencia', 'cnae', 'tarifa', 'butlletins', 'tensio'
                    ]

                    polissa = O.GiscedataPolissa.read(
                        polissa_id[0], fields_to_read, context_glob
                    )
                    if polissa['tensio']:
                        o_tensio = polissa['tensio'] / 1000.0
                    o_potencia = polissa['potencia']
                    if polissa['cnae']:
                        o_cnae = polissa['cnae'][1]
                    # Mirem si té l'actualització dels butlletins
                    if polissa.get('butlletins', []):
                        butlleti = O.GiscedataButlleti.read(
                            polissa['butlletins'][-1], ['pot_max_admisible']
                        )
                        o_pot_ads = butlleti['pot_max_admisible']
                    comptador_actiu = get_comptador(
                        self.connection, polissa['id'], self.year)
                    if comptador_actiu:
                        comptador_actiu = comptador_actiu[0]
                        comptador = O.GiscedataLecturesComptador.read(
                            comptador_actiu, ['cini', 'tg']
                        )
                        if not comptador['cini']:
                            comptador['cini'] = ''
                        if comptador.get('tg', False):
                            o_equip = 'SMT'
                        elif re.findall(CINI_TG_REGEXP, comptador['cini']):
                            o_equip = 'SMT'
                        else:
                            o_equip = 'MEC'
                    if polissa['tarifa']:
                        o_cod_tfa = self.get_codi_tarifa(polissa['tarifa'][1])
                else:
                    # Si no trobem polissa activa, considerem
                    # "Contrato no activo (CNA)"
                    o_equip = 'CNA'
                    o_estat_contracte = 1

                #energies consumides
                o_anual_activa = cups['cne_anual_activa'] or 0.0
                o_anual_reactiva = cups['cne_anual_reactiva'] or 0.0
                o_any_incorporacio = self.year + 1
                self.output_q.put([
                    o_nom_node,
                    o_utmx,
                    o_utmy,
                    o_utmz,
                    o_cnae,
                    o_equip,
                    o_cod_tfa,
                    o_zona,
                    o_name,
                    o_codi_r1,
                    o_codi_ine,
                    o_codi_prov,
                    o_connexio,
                    o_tensio,
                    o_estat_contracte,
                    o_potencia,
                    o_potencia_facturada,
                    o_pot_ads or o_potencia,
                    o_anual_activa,
                    o_anual_reactiva,
                    o_any_incorporacio
                ])
            except:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import traceback

from libcnmc.res_4667.utils import get_resum_any_id
from libcnmc.utils import format_f
from libcnmc.core import MultiprocessBased


class RES(MultiprocessBased):
    """
    Class to generate F2 of 4667
    """

    def __init__(self, **kwargs):
        """
        Class constructor
        
        :param kwargs: 
        :type kwargs: dict 
        """

        self.year = kwargs.pop("year")

        self.years = []

        self.limite_empresa = {}
        self.demanda_empresa_p0 = {}
        self.inc_demanda_empresa_prv = {}
        self.frri = {}
        self.vpi_superado_prv = {}
        self.vol_total_inv_prv = {}
        self.ayudas_prv = {}
        self.financiacion_prv = {}
        self.vpi_retribuible_prv = {}
        self.num_proyectos = {}
        self.vol_total_inv_bt_prv = {}

        super(RES, self).__init__(**kwargs)

    def get_sequence(self):
        """
        Generates the sequence of ids to make the report

        :return: List of ids
        :rtype: list
        """

        ids_resums = get_resum_any_id(self.connection, self.year)
        return ids_resums

    def consumer(self):
        """
        Generates the line of the file
        :return: Line 
        :rtype: str
        """

        O = self.connection
        fields_to_read = [
            "anyo", "limit_empresa", "demanda_empresa_p0", "inc_demanda",
            "frri", "vpi_sup", "volum_total_inv", "ajudes_prev", "financiacio",
            "vpi_retribuible_prv", "n_projectes", "voltotal_inv_bt_prv",
            "vol_total_inv_gr_prv", "vol_total_inv_prv_prtr",
            "ayudas_prv_prtr", "financiacion_prv_prtr",
            "vpi_retribuible_prv_prtr", "num_proyectos_prtr"
        ]

        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)

                resumen = O.GiscedataCnmcResum_any.read(item, fields_to_read)
                vpi_superado_prv = str(resumen["vpi_sup"]).upper()
                demanda_empresa_p0 = format_f(resumen["demanda_empresa_p0"], 3)
                if len(demanda_empresa_p0) > 8:
                    demanda_empresa_p0 = format_f(
                        resumen["demanda_empresa_p0"], 1
                    )
                output = [
                    resumen["anyo"],
                    format_f(resumen["limit_empresa"], 2) or "0.00",
                    demanda_empresa_p0 or "0.00",
                    format_f(resumen["inc_demanda"], 3) or "0.00",
                    format_f(resumen["frri"], 3) or "0.00",
                    vpi_superado_prv,
                    format_f(resumen["volum_total_inv"], 2) or "0.00",
                    format_f(resumen["ajudes_prev"], 2) or "0.00",
                    format_f(resumen["financiacio"], 2) or "0.00",
                    format_f(resumen["vpi_retribuible_prv"], 2) or "0.00",
                    resumen["n_projectes"],
                    format_f(resumen["voltotal_inv_bt_prv"], 2) or "0.00",
                    format_f(resumen["vol_total_inv_gr_prv"], 2) or "0.00",
                    format_f(resumen["vol_total_inv_prv_prtr"], 2) or "0.00",
                    format_f(resumen["ayudas_prv_prtr"], 2) or "0.00",
                    format_f(resumen["financiacion_prv_prtr"], 2) or "0.00",
                    format_f(resumen["vpi_retribuible_prv_prtr"], 2) or "0.00",
                    resumen["num_proyectos_prtr"] or "0",
                ]
                self.output_q.put(output)

            except Exception:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



from __future__ import absolute_import
from collections import OrderedDict

from libcnmc.models.cnmcmodel import CNMCModel
from libcnmc.models.fields import String, Integer, Decimal
import string
import re


class F2Res4666(CNMCModel):
    """
        Class for second file of resolution 4666(BT) (Trams BT)
    """

    schema = OrderedDict([
        ('identificador', String()),
        ('cini', String()),
        ('origen', String()),
        ('destino', String()),
        ('codigo_ccuu', String()),
        ('codigo_ccaa_1', Integer()),
        ('codigo_ccaa_2', Integer()),
        ('participacion', String()),
        ('fecha_aps', String()),
        ('fecha_baja', String()),
        ('numero_circuitos', Integer()),
        ('numero_conductores', Integer()),
        ('nivel_tension', Decimal(3)),
        ('longitud', Decimal(3)),
        ('intensidad_maxima', Decimal(3)),
        ('seccion', Decimal(3)),
        ('capacidad', Decimal(3)),
        ('estado', Integer())
    ])

    def ref_or_regulatori(self, trams_at_prefix, trams_bt_prefix):
        return re.sub('^{}'.format(trams_bt_prefix), '', self.store.identificador)

    def __cmp__(self, other):
        comp_fields = [
            'longitud', 'cini', 'codigo_ccuu', 'nivel_tension', 'fecha_aps'
        ]
        if self.diff(other, comp_fields):
            return True
        else:
            return False








# -*- coding: utf-8 -*-
from datetime import datetime
import traceback
from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import TARIFAS_BT, TARIFAS_AT


class FA4(StopMultiprocessBased):
    def __init__(self, **kwargs):
        super(FA4, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.reducir_cups = kwargs.get("reducir_cups", False)
        self.report_name = 'FA4 - CUPS-CT'
        self.base_object = 'CTS i CUPS'
        mod_all_year = self.connection.GiscedataPolissaModcontractual.search(
            [
                ("data_inici", "<=", "{}-01-01".format(self.year)),
                ("data_final", ">=", "{}-12-31".format(self.year)),
                ("tarifa.name", 'not ilike', '%RE%'),
                ('polissa_id.state', 'in', ['tall', 'activa', 'baixa'])
            ], 0, 0, False, {"active_test": False}
                                )
        mods_ini = self.connection.GiscedataPolissaModcontractual.search(
            [
                ("data_inici", ">=", "{}-01-01".format(self.year)),
                ("data_inici", "<=", "{}-12-31".format(self.year)),
                ("tarifa.name", 'not ilike', '%RE%'),
                ('polissa_id.state', 'in', ['tall', 'activa', 'baixa'])
            ], 0, 0, False, {"active_test": False}
        )
        mods_fi = self.connection.GiscedataPolissaModcontractual.search(
            [
                ("data_final", ">=", "{}-01-01".format(self.year)),
                ("data_final", "<=", "{}-12-31".format(self.year)),
                ("tarifa.name", 'not ilike', '%RE%'),
                ('polissa_id.state', 'in', ['tall', 'activa', 'baixa'])
            ], 0, 0, False, {"active_test": False}
        )

        self.generate_derechos = kwargs.pop("derechos", False)
        self.modcons_in_year = set(mods_fi + mods_ini + mod_all_year)

    def get_derechos(self, tarifas, years):
        """
        Returns a list of CUPS with derechos

        :param tarifas: Lis of tarifas of the polissas that are in the CUPS
        :param years: Number of years to search back
        :return: List of ids of the
        """

        O = self.connection

        polisses_baixa_id = O.GiscedataPolissa.search(
            [
                ("data_baixa", "<=", "{}-12-31".format(self.year - 1)),
                ("data_baixa", ">", "{}-01-01".format(self.year - years)),
                ("tarifa", "in", tarifas)
            ],
            0, 0, False, {'active_test': False}
        )

        cups_polisses_baixa = [x["cups"][0] for x in O.GiscedataPolissa.read(
            polisses_baixa_id, ["cups"]
        )]

        cups_derechos = O.GiscedataCupsPs.search(
            [
                ("id", "in", cups_polisses_baixa),
                ("polissa_polissa", "=", False)
            ]
        )

        polissa_eliminar_id = O.GiscedataPolissaModcontractual.search(
            [
                ("cups", "in", cups_derechos),
                '|', ("data_inici", ">", "{}-01-01".format(self.year)),
                ("data_final", ">", "{}-01-01".format(self.year))
            ],
            0, 0, False, {'active_test': False}
        )

        cups_eliminar_id = [x["cups"][0] for x in
                            O.GiscedataPolissaModcontractual.read(
                                polissa_eliminar_id, ["cups"]
                            )]

        cups_derechos = list(set(cups_derechos) - set(cups_eliminar_id))

        return cups_derechos

    def get_sequence(self):
        data_ini = '%s-01-01' % (self.year + 1)
        # data_baixa = '%s-12-31' % self.year
        search_params = [('active', '=', True),
                         '|',
                         ('create_date', '<', data_ini),
                         ('create_date', '=', False)]

        ret_cups_ids = self.connection.GiscedataCupsPs.search(
            search_params, 0, 0, False, {'active_test': False})

        ret_cups_tmp = self.connection.GiscedataCupsPs.read(
            ret_cups_ids, ["polisses"]
        )
        ret_cups = []

        for cups in ret_cups_tmp:
            if set(cups['polisses']).intersection(self.modcons_in_year):
                ret_cups.append(cups["id"])

        if self.generate_derechos:
            cups_derechos_bt = self.get_derechos(TARIFAS_BT, 2)
            cups_derechos_at = self.get_derechos(TARIFAS_AT, 4)
            return list(set(ret_cups + cups_derechos_at + cups_derechos_bt))
        else:
            return ret_cups

    def get_cini(self, et):
        o = self.connection
        valor = ''
        if et:
            cts = o.GiscedataCts.search([('name', '=', et)])
            if cts:
                cini = o.GiscedataCts.read(cts[0], ['cini'])
                valor = cini['cini']
        return valor

    def consumer(self):
        """
        Consumer function that executes for each value returned by get_sequence

        :return: None
        """

        o = self.connection
        fields_to_read = [
            'name', 'et', 'polisses', 'id'
        ]
        while True:
            try:
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                cups = o.GiscedataCupsPs.read(
                    item, fields_to_read
                )

                if self.reducir_cups:
                    o_cups = cups['name'][:20]
                else:
                    o_cups = cups['name']
                o_cini = self.get_cini(cups['et'])
                if not o_cini:
                    o_cini = 'False'
                o_codi_ct = cups['et']
                self.output_q.put([
                    o_cups,         # CUPS
                    o_cini,         # CINI
                    o_codi_ct       # CODIGO SUBESTACION
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



# -*- coding: utf-8 -*-
import click
from libcnmc.utils import N_PROC
from libcnmc.core.backend import OOOPFactory
from datetime import datetime


@click.group()
def cnmc_4667():
    pass


# CSV 4667
def csv_4667(obj, **kwargs):
    """
    Starts the proces to generate the 4667

    :param obj: Object of the process to generate
    :param kwargs: kwargs to pass to the process
    :type kwargs: dict(str,str)
    :return: None
    :rtype: None
    """

    con = OOOPFactory(
        dbname=kwargs['database'], user=kwargs['user'],
        pwd=kwargs['password'], port=kwargs['port'],
        uri=kwargs['server'])
    proc = obj(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=con,
        num_proc=kwargs['num_proc'],
        codi_r1=kwargs['codi_r1'],
        year=kwargs['year'],
    )
    proc.calc()


@cnmc_4667.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4667_res(**kwargs):
    """
    Click entry to generate the resumen of 4667 report

    :param kwargs: Options to generate the file 
    :return: None 
    """

    from libcnmc.res_4667 import RES
    csv_4667(RES, **kwargs)


@cnmc_4667.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4667_macro(**kwargs):
    """
    Click entry to generate the proyectos of 4667 report

    :param kwargs: Options to generate the file 
    :return: None 
    """

    from libcnmc.res_4667 import MACRO
    csv_4667(MACRO, **kwargs)


@cnmc_4667.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4667_pro(**kwargs):
    """
    Click entry to generate the proyectos of 4667 report

    :param kwargs: Options to generate the file 
    :return: None 
    """

    from libcnmc.res_4667 import PRO
    csv_4667(PRO, **kwargs)


@cnmc_4667.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4667_resccaa(**kwargs):
    """
    Click entry to generate the resumen(CCAA) of 4667 report

    :param kwargs: Options to generate the file 
    :return: None 
    """

    from libcnmc.res_4667 import RESCCAA
    csv_4667(RESCCAA, **kwargs)


@cnmc_4667.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4667_fia(**kwargs):
    """
    Click entry to generate the fiabilitat of 4667 report

    :param kwargs: Options to generate the file 
    :return: None 
    """

    from libcnmc.res_4667 import FIA
    csv_4667(FIA, **kwargs)


@cnmc_4667.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4667_des(**kwargs):
    """
    Click entry to generate the maquines of 4667 report

    :param kwargs: Options to generate the file 
    :return: None 
    """

    from libcnmc.res_4667 import DES
    csv_4667(DES, **kwargs)


@cnmc_4667.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4667_maq(**kwargs):
    """
    Click entry to generate the maquines of 4667 report

    :param kwargs: Options to generate the file 
    :return: None 
    """

    from libcnmc.res_4667 import MAQ
    csv_4667(MAQ, **kwargs)


@cnmc_4667.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4667_pos(**kwargs):
    """
    Click entry to generate the posiciones of 4667 report

    :param kwargs: Options to generate the file 
    :return: None 
    """

    from libcnmc.res_4667 import POS
    csv_4667(POS, **kwargs)


@cnmc_4667.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4667_otros(**kwargs):
    """
    Click entry to generate the Otros of 4667 report

    :param kwargs: Options to generate the file 
    :return: None 
    """

    from libcnmc.res_4667 import Otros
    csv_4667(Otros, **kwargs)


@cnmc_4667.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4667_lbt(**kwargs):
    """
    Click entry to generate the LBT of 4667 report

    :param kwargs: Options to generate the file 
    :return: None 
    """

    from libcnmc.res_4667 import LBT
    csv_4667(LBT, **kwargs)


@cnmc_4667.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4667_lat(**kwargs):
    """
    Click entry to generate the LAT file of 4667

    :param kwargs: Options to generate the file  
    :return: None 
    """

    from libcnmc.res_4667 import LAT
    csv_4667(LAT, **kwargs)

@cnmc_4667.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4667_ct(**kwargs):
    """
    Click entry to generate the resumen of 4667 report

    :param kwargs: Options to generate the file 
    :return: None 
    """

    from libcnmc.res_4667 import CT
    csv_4667(CT, **kwargs)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import traceback

from libcnmc.res_4667.utils import get_resum_any_id
from libcnmc.core import MultiprocessBased
from libcnmc.utils import format_f


class MACRO(MultiprocessBased):
    """
    Class to generate F2 of 4667
    """

    def __init__(self, **kwargs):
        """
        Class constructor
        :param kwargs: 
        """

        self.year = kwargs.pop('year')
        super(MACRO, self).__init__(**kwargs)

    def get_sequence(self):
        """
        Generates the sequence of ids to make the report

        :return: List of ids
        :rtype: list
        """
        ids_resum = get_resum_any_id(self.connection, self.year)

        return ids_resum

    def consumer(self):
        """
        Generates the line of the file
        :return: Line 
        :rtype: str
        """

        O = self.connection
        fields_to_read = [
            "anyo", "macro_crec_pib", "macro_pib_prev", "macro_limite_sector",
            "macro_inc_demanda_sector"
        ]

        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)

                macro = O.GiscedataCnmcResum_any.read(item, fields_to_read)
                output = [
                    macro["anyo"],
                    format_f(float(macro["macro_crec_pib"]), 4),
                    format_f(float(macro["macro_pib_prev"]), 2) or "0.00",
                    format_f(float(macro["macro_limite_sector"]), 2) or "0.00",
                    format_f(float(macro["macro_inc_demanda_sector"]), 2) or "0.00"
                ]
                self.output_q.put(output)

            except Exception:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC Centres Transformadors
"""
from __future__ import absolute_import
from datetime import datetime
import traceback

from libcnmc.utils import format_f
from libcnmc.core import StopMultiprocessBased

OPERACIO = {
    'Reserva': '0',
    'Operativo': '1',
}

class FB2_1(StopMultiprocessBased):
    """
    Class that generates the CT file of the 4666
    """
    def __init__(self, **kwargs):
        """
        Class constructor
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: CT
        """
        super(FB2_1, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'CTS'
        self.report_name = 'Formulario B2.1: Máquinas en Centros de Transformación'
        self.compare_field = ""
        self.extended = kwargs.get("extended", False)

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consummer
        :return: List of ids
        :rtype: list[int]
        """

        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        search_params = [
            ('reductor', '=', False),
            ('id_estat.cnmc_inventari', '=', True),
            ('propietari', '=', True),
        ]
        search_params += ['|', ('data_pm', '=', False),
                          ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                          ('data_baixa', '=', False)
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                          ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataTransformadorTrafo.search(
            search_params, 0, 0, False, {'active_test': False})

    def get_operacio(self, id_estat):

        o = self.connection
        operacio = o.GiscedataTransformadorEstat.read(
            id_estat[0], ['operacio']
        )['operacio']
        return operacio


    def consumer(self):
        """
        Method that generates the csv file
        :return: List of arrays
        """
        o = self.connection
        fields_to_read = [
            'ct', 'name', 'cini', 'potencia_nominal',
            'data_pm', 'id_estat'
        ]
        while True:
            try:
                # generar linies
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                trafo = o.GiscedataTransformadorTrafo.read(
                    item, fields_to_read
                )

                o_ct = trafo['ct'] and trafo['ct'][1] or ''
                o_cini = trafo['cini'] or ''
                o_maquina = trafo['name']
                o_pot = format_f(
                    float(trafo['potencia_nominal']),
                    decimals=3
                )

                tmp_date = datetime.strptime(
                    trafo['data_pm'], '%Y-%m-%d')
                o_any = tmp_date.strftime('%Y')

                id_estat = trafo['id_estat']

                if id_estat:
                    o_operacio = self.get_operacio(id_estat)
                else:
                    o_operacio = 0

                if o_cini:
                    if o_cini[1] == '2' and o_cini[2] == '7' and o_cini[5] == '2' and o_cini[7] == '1':
                        o_operacio = 0

                self.output_q.put([
                    o_ct,           # CT
                    o_maquina,      # MAQUINA
                    o_cini,         # CINI
                    o_pot,          # POT MAQUINA
                    o_any,          # AÑO INFORMACION
                    o_operacio     # OPERACION
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



from LAT import LAT
from LBT import LBT
from CT import CT
from DES import DES
from FIA import FIA
from MACRO import MACRO
from MAQ import MAQ
from Otros import Otros
from POS import POS
from PRO import PRO
from RES import RES
from RES_CCAA import RESCCAA



# -*- coding: utf-8 -*-
from datetime import datetime
import traceback
from libcnmc.utils import format_f, convert_srid, get_srid
from libcnmc.core import StopMultiprocessBased
from shapely import wkt


class F15(StopMultiprocessBased):
    """
    Class to generate the F15 (Celdas)
    """

    def __init__(self, **kwargs):
        """
        Class constructor

        :param year: Year to generate
        :type year: int
        :param codi_r1:
        :type codi_r1:str
        """

        super(F15, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.report_name = 'F15 - Cel·les'
        self.base_object = 'Cel·les'
        self.cod_dis = 'R1-{}'.format(self.codi_r1[-3:])

    def get_sequence(self):
        """
        Generates the sequence of ids to pass to the consume function

        :return: List of ids to generate the
        :rtype: list(int)
        """
        search_params = [
            ("installacio", "like", "giscedata.at.suport"),
            ("tipus_element.codi_cnmc", "!=", "T")
        ]
        return self.connection.GiscedataCellesCella.search(search_params)

    def get_node_vertex(self, element_name):
        """
        Return the node and the (X,Y) of the Cel·la retrieving this values from
        the Blocs models
        :param element_name: Name of the Cel·la
        :rtype element_name: str
        :return: The Node name and the (X,Y) of the Cel·la
        :rtype: (str or bool, (float, float) or bool)
        """
        o = self.connection
        node = ''
        vertex = False
        bloc_id = []
        model_ok = False
        if element_name:
            # Search on the diferent models
            models = [
                o.GiscegisBlocsInterruptorat,
                o.GiscegisBlocsFusiblesat,
                o.GiscegisBlocsSeccionadorat,
                o.GiscegisBlocsSeccionadorunifilar
            ]
            for model in models:
                bloc_id = model.search([('codi', '=', element_name)])
                if bloc_id:
                    model_ok = model
                    break
            if bloc_id:
                bloc = model_ok.read(
                    bloc_id[0], ['node', 'vertex']
                )
                if bloc.get('node', False):
                    node = bloc['node'][1]
                if bloc.get('vertex', False):
                    v = o.GiscegisVertex.read(bloc['vertex'][0], ['x', 'y'])
                    vertex = (v['x'], v['y'])

        return node, vertex

    def obtenir_camps_linia(self, installacio):
        """
        Gets the data of the line where the cel·la is placed

        :param installacio: Cel·la placement
        :return: Municipi, provincia, tensio of the line
        :rtype: dict
        """

        o = self.connection
        id_tram = int(installacio.split(',')[1])

        tram = o.GiscedataAtSuport.read(id_tram, ['linia'])
        linia_id = tram['linia']
        fields_to_read = [
            'municipi', 'provincia', 'tensio'
        ]
        linia = o.GiscedataAtLinia.read(int(linia_id[0]), fields_to_read)
        municipi = ''
        provincia = ''
        id_municipi = linia['municipi'][0]
        id_provincia = linia['provincia'][0]
        tensio = format_f(float(linia['tensio']) / 1000.0, decimals=3)

        if id_municipi and id_provincia:
            provincia = o.ResCountryState.read(id_provincia, ['code'])['code']
            municipi_dict = o.ResMunicipi.read(id_municipi, ['ine', 'dc'])
            municipi = '{0}{1}'.format(municipi_dict['ine'][-3:],
                                       municipi_dict['dc'])

        res = {
            'municipi': municipi,
            'provincia': provincia,
            'tensio': tensio
        }

        return res

    def get_node_vertex_tram(self, element_name):
        o = self.connection
        tram_name = ""
        if element_name:
            # Search on the diferent models
            models = [
                o.GiscegisBlocsInterruptorat, o.GiscegisBlocsFusiblesat,
                o.GiscegisBlocsSeccionadorat, o.GiscegisBlocsSeccionadorunifilar
            ]
            for model in models:
                bloc_id = model.search([('codi', '=', element_name)])
                if bloc_id:
                    bloc_id = bloc_id[0]
                    bloc = model.read(bloc_id, ['node', 'vertex'])
                    v = o.GiscegisVertex.read(bloc['vertex'][0], ['id'])
                    polver_ids = o.GiscegisPolylineVertex.search(
                        [('vertex', '=', v['id'])]
                    )
                    if polver_ids:
                        poly_id = o.GiscegisPolyline.search(
                            [('vertex_ids', 'in', polver_ids[0])]
                        )[0]
                        edge_id = o.GiscegisEdge.search(
                            [('polyline', '=', poly_id)]
                        )[0]
                        linktemplate = o.GiscegisEdge.read(
                            edge_id, ['id_linktemplate']
                        )['id_linktemplate']
                        tram_id = o.GiscedataAtTram.search(
                            [('name', '=', linktemplate)]
                        )
                        tram_name = o.GiscedataAtTram.read(
                            tram_id, ['name']
                        )[0]['name']
                        return "A{0}".format(tram_name)
        return ""

    def consumer(self):
        """
        Function that generates each line of the file

        :return: None
        """

        o = self.connection
        fields_to_read = [
            'installacio', 'cini', 'propietari', 'name', 'tensio', 'node_id',
            'geom', 'tram_id'
        ]
        while True:
            try:
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                cella = o.GiscedataCellesCella.read(
                    item, fields_to_read
                )
                o_fiabilitat = cella['name']
                o_cini = cella['cini']
                o_prop = int(cella['propietari'])
                o_tram = ""
                if cella['tram_id']:
                    o_tram = "A{0}".format(
                        o.GiscedataAtTram.read(
                            cella['tram_id'][0], ['name']
                        )['name']
                    )
                else:
                    o_tram = self.get_node_vertex_tram(o_fiabilitat)

                if cella.get("node_id"):
                    o_node = cella["node_id"][1]
                    vertex = wkt.loads(cella["geom"]).coords[0]
                else:
                    o_node, vertex = self.get_node_vertex(o_fiabilitat)
                o_node = o_node.replace('*', '')

                dict_linia = self.obtenir_camps_linia(cella['installacio'])
                o_municipi = dict_linia.get('municipi')
                o_provincia = dict_linia.get('provincia')
                if cella['tensio']:
                    tensio = o.GiscedataTensionsTensio.read(
                        cella['tensio'][0], ['tensio']
                    )
                    o_tensio = format_f(int(tensio['tensio'])/1000.0, decimals=3)
                else:
                    o_tensio = dict_linia.get('tensio')

                o_any = self.year
                x = ''
                y = ''
                z = ''
                if vertex:
                    res_srid = convert_srid(get_srid(o), vertex)
                    x = format_f(res_srid[0], decimals=3)
                    y = format_f(res_srid[1], decimals=3)
                self.output_q.put([
                    o_node,         # NUDO
                    o_fiabilitat,   # ELEMENTO FIABILIDAD
                    o_tram,         # TRAMO
                    o_cini,         # CINI
                    x,              # X
                    y,              # Y
                    z,              # Z
                    o_municipi,     # MUNICIPIO
                    o_provincia,    # PROVINCIA
                    o_tensio,       # NIVEL TENSION
                    self.cod_dis,   # CODIGO DISTRIBUIDORA
                    o_prop,         # PROPIEDAD
                    o_any           # AÑO INFORMACION
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from datetime import datetime
import traceback
from libcnmc.core import StopMultiprocessBased


def get_ti_name(connection, ti_id):
    """
    Get the name of the ti from the id
    
    :param connection: OpenERP conection
    :param ti_id: id of the TI
    :type ti_id: tuple
    :return: Name of the TI
    :rtype: str
    """
    if ti_id:
        model_ti = connection.GiscedataTipusInstallacio
        codigo_ccuu = model_ti.read(ti_id[0], ['name'])['name']
        return codigo_ccuu
    else:
        return ""


class ModCts(StopMultiprocessBased):
    """
    Class that generates the modification file of CT
    """

    def __init__(self, **kwargs):
        """
        Class constructor
        
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: None
        :rtype: None
        """

        super(ModCts, self).__init__(**kwargs)
        self.explain = kwargs.pop('explain', False)
        self.year = kwargs.pop('year', datetime.now().year - 1)

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consumer
        
        :return: Ids to generate
        :rtype: list
        """

        search_params = [('id_installacio.name', '!=', 'SE')]
        data_pm = '{0}-01-01'.format(self.year + 1)
        data_baixa = '{0}-01-01'.format(self.year)
        search_params += [('propietari', '=', True),
                          '|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|',
                          '&', ('data_baixa', '>', data_baixa),
                               ('ct_baixa', '=', True),
                          '|',
                               ('data_baixa', '=', False),
                               ('ct_baixa', '=', False)
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataCts.search(
            search_params, 0, 0, False, {'active_test': False})

    def consumer(self):
        """
        Method that generates the csb file
        
        :return: None
        :rtype: None
        """

        O = self.connection

        while True:
            try:
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)

                fields_to_read = [
                    "4666_entregada_2017", "4771_entregada", "name"
                ]
                ct = O.GiscedataCts.read(item, fields_to_read)
                ti_old = ct["4771_entregada"].get("codigo_tipo_ct", "")
                ti = ct["4666_entregada_2017"].get("codigo_ccuu", "")

                if self.explain:
                    if ti_old:
                        if ti_old == ti:
                            print("[CT] id:{} ti_old:{} == ti:{}".format(ct["id"], ti_old, ti))
                    else:
                        print("[CT] id:{} has no old ti".format(ct["id"]))

                if ti_old and ti_old != ti:
                    output = [
                        ct["name"],
                        "",
                        ti_old,
                        ti
                    ]
                    self.output_q.put(output)
                    self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()


class ModFia(StopMultiprocessBased):
    """
    Class that generates the modifications of fiabilidad(7)
    """

    def __init__(self, **kwargs):
        """
        Class constructor
        
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: None
        :rtype: None
        """

        super(ModFia, self).__init__(**kwargs)
        self.explain = kwargs.pop("explain", False)
        self.year = kwargs.pop('year', datetime.now().year - 1)

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consumer
        
        :return: ids
        :rtype: list
        """

        search_params = [('inventari', '=', 'fiabilitat')]
        data_pm = '{0}-01-01' .format(self.year + 1)
        data_baixa = '{0}-01-01'.format(self.year)
        search_params += [('propietari', '=', True),
                          '|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                               ('data_baixa', '=', False)
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        search_params += [("cini", "not like", "I28")]
        return self.connection.GiscedataCellesCella.search(
            search_params, 0, 0, False, {'active_test': False})

    def consumer(self):
        """
        Method that generates the csv file
        
        :return: None
        :rtype: None
        """

        O = self.connection
        while True:
            try:
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)

                fields_to_read = [
                    "4666_entregada_2017", "4771_entregada", "name"
                ]
                cll = O.GiscedataCellesCella.read(item, fields_to_read)

                ti_old = cll["4771_entregada"].get("codigo_tipo_inst", "")
                ti = cll["4666_entregada_2017"].get("codigo_ccuu", "")

                if self.explain:
                    if ti_old:
                        if ti_old == ti:
                            print("[FIA] id:{} ti_old:{} == ti:{}".format(cll["id"], ti_old, ti))
                    else:
                        print("[FIA] id:{} has no old ti".format(cll["id"]))

                if ti_old and ti_old != ti:
                    output = [
                        cll["name"],
                        "",
                        ti_old,
                        ti
                    ]
                    self.output_q.put(output)
                    self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()


class ModLat(StopMultiprocessBased):
    """
    Class that generates modifications of the LAT(1)
    """

    def __init__(self, **kwargs):
        """
        Class constructor
    
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: None
        :rtype: None
        """

        super(ModLat, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.explain = kwargs.pop("explain", False)

        id_res_like = self.connection.ResConfig.search(
            [('name', '=', 'giscegis_btlike_layer')])
        if id_res_like:
            self.layer = self.connection.ResConfig.read(
                id_res_like, ['value'])[0]['value']

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consumer
       
        :return: List of ids
        """

        search_params = [('propietari', '=', True)]
        obj_lat = self.connection.GiscedataAtLinia
        ids = obj_lat.search(search_params, 0, 0, False, {'active_test': False})

        return ids

    def consumer(self):
        """
        Method that generates the csb file
        
        :return: None
        :rtype: None
        """

        O = self.connection

        data_pm_limit = '{0}-01-01'.format(self.year + 1)
        data_baixa = '{0}-01-01'.format(self.year)

        static_search_params = [
            ('propietari', '=', True),
            '|', ('data_pm', '=', False), ('data_pm', '<', data_pm_limit),
            '|', ('data_baixa', '=', False), ('data_baixa', '>', data_baixa)
            ]

        # Revisem que si està de baixa ha de tenir la data informada.
        static_search_params += [
            '|',
            '&', ('active', '=', False), ('data_baixa', '!=', False),
            ('active', '=', True)
        ]
        while True:
            try:
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)

                linia = O.GiscedataAtLinia.read(item, ['trams'])

                search_params = [('linia', '=', linia['id'])]
                search_params += static_search_params
                ids = O.GiscedataAtTram.search(
                    search_params, 0, 0, False, {'active_test': False})
                fields_to_read_tram = [
                    "4666_entregada_2017",
                    "4771_entregada",
                    "name"
                ]

                for tram in O.GiscedataAtTram.read(ids, fields_to_read_tram):
                    ti_old = tram["4771_entregada"].get("codigo_tipo_linea", "")
                    ti = tram["4666_entregada_2017"].get("codigo_ccuu", "")

                    if self.explain:
                        if ti_old:
                            if ti_old == ti:
                                print("[LAT] id:{} ti_old:{} == ti:{}".format(tram["id"], ti_old, ti))
                        else:
                            print("[LAT] id:{} has no old ti".format(tram["id"]))

                    if ti_old and ti_old != ti:
                        output = [
                            'A{0}'.format(tram["name"]),
                            "",
                            ti_old,
                            ti
                        ]
                        self.output_q.put(output)
                        self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()


class ModLbt(StopMultiprocessBased):
    """
    Class that generates the modifications of LBT(2)
    """

    def __init__(self, **kwargs):
        """
        Class constructor
    
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: CT
        """

        super(ModLbt, self).__init__(**kwargs)
        self.year = kwargs.pop("year", datetime.now().year - 1)
        self.explain = kwargs.pop("explain", False)

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consumer
        
        :return: Ids
        :rtype: list
        """

        data_pm = '{0}-01-01'.format(self.year + 1)
        data_baixa = '{0}-01-01'.format(self.year)
        search_params = []
        search_params += [('propietari', '=', True),
                          '|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|',
                          '&', ('data_baixa', '>', data_baixa),
                               ('baixa', '=', True),
                          '|',
                               ('data_baixa', '=', False),
                               ('baixa', '=', False)
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataBtElement.search(
            search_params, 0, 0, False, {'active_test': False})

    def consumer(self):
        """
        Method that generates the csb file

        :return: None
        :rtype: None
        """

        O = self.connection
        count = 0
        fields_to_read = [
            "4666_entregada_2017", "4771_entregada", "name"
        ]
        while True:
            try:
                count += 1
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)

                linia = O.GiscedataBtElement.read(item, fields_to_read)
                ti_old = linia["4771_entregada"].get("codigo_tipo_linea", "")
                ti = linia["4666_entregada_2017"].get("codigo_ccuu", "")

                if self.explain:
                    if ti_old:
                        if ti_old == ti:
                            print("[LBT] id:{} ti_old:{} == ti:{}".format(linia["id"], ti_old, ti))
                    else:
                        print("[LBT] id:{} has no old ti".format(linia["id"]))

                if ti_old and ti_old != ti:
                    output = [
                        "B{}".format(linia["name"]),
                        "",
                        ti_old,
                        ti
                    ]

                    self.output_q.put(output)
                    self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()


class ModMaq(StopMultiprocessBased):
    """
    Class that generates the modifications of Maquinas/Transofrmadores(5)
    """

    def __init__(self, **kwargs):
        """
        Class constructor
    
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: None
        :rtype: None
        """
        super(ModMaq, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.explain = kwargs.pop("explai", False)

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consumer
        
        :return: Ids of maquines
        :rtype: list
        """

        data_pm = '{0}-01-01'.format(self.year + 1)
        data_baixa = '{0}-01-01'.format(self.year)
        search_params = [
            ('propietari', '=', True),
            '|', ('data_pm', '=', False),
            ('data_pm', '<', data_pm),
            '|',
            '&', ('data_baixa', '>', data_baixa),
                 ('baixa', '=', True),
            '|',
                 ('data_baixa', '=', False),
                 ('baixa', '=', False)
        ]

        # Transformadors reductors
        search_params_reductor = search_params + [
            ('id_estat.cnmc_inventari', '=', True),
            ('reductor', '=', True)]

        # Transformadors
        search_params_transformadors = search_params + [
            ('id_estat.cnmc_inventari', '=', True),
            ('localitzacio.code', '=', '1'),
            ('id_estat.codi', '!=', '1')]

        # search_params_reductor += search_params
        ids_reductor = self.connection.GiscedataTransformadorTrafo.search(
            search_params_reductor, 0, 0, False, {'active_test': False})

        ids_transformadors = self.connection.GiscedataTransformadorTrafo.search(
            search_params_transformadors, 0, 0, False, {'active_test': False})
        return list(set(ids_reductor + ids_transformadors))

    def consumer(self):
        """
        Method that generates each line of the CSV
        
        :return: None
        :rtype: None
        """

        O = self.connection
        fields_to_read = ["4666_entregada_2017", "4771_entregada", "name"]

        while True:
            try:
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)

                trafo = O.GiscedataTransformadorTrafo.read(item, fields_to_read)
                ti_old = trafo["4771_entregada"].get("codigo_tipo_maquina", "")
                ti = trafo["4666_entregada_2017"].get("codigo_ccuu", "")

                if self.explain:
                    if ti_old:
                        if ti_old == ti:
                            print("[MAQ] id:{} ti_old:{} == ti:{}".format(trafo["id"], ti_old, ti))
                    else:
                        print("[MAQ] id:{} has no old ti".format(trafo["id"]))

                if ti_old and ti != ti_old:
                    output = [
                        trafo["name"],
                        "",
                        ti_old,
                        ti
                    ]
                    self.output_q.put(output)
                    self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()


class ModPos(StopMultiprocessBased):
    """
    Class that generates the modifications of POS/Interruptores(4)
    """

    def __init__(self, **kwargs):
        """
        Class constructor
        
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: CT
        """

        super(ModPos, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.explain = kwargs.pop("explain", False)

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consumer
        
        :return: Ids
        :rtype: list
        """

        search_params = [('interruptor', '=', '2')]
        data_pm = '{0}-01-01'.format(self.year + 1)
        data_baixa = '{0}-01-01'.format(self.year)
        search_params += [('propietari', '=', True),
                          '|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                               ('data_baixa', '=', False),
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataCtsSubestacionsPosicio.search(
            search_params, 0, 0, False, {'active_test': False})

    def consumer(self):
        """
        Method that generates the csv file

        :return: None
        :rtype: None
        """

        O = self.connection
        fields_to_read = [
            "4666_entregada_2017",
            "4771_entregada",
            "name"
        ]

        while True:
            try:
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                pos = O.GiscedataCtsSubestacionsPosicio.read(item, fields_to_read)

                ti_old = pos["4771_entregada"].get("codigo_tipo_ct", "")
                ti = pos["4666_entregada_2017"].get("codigo_ccuu", "")

                if self.explain:
                    if ti_old:
                        if ti_old == ti:
                            print("[POS] id:{} ti_old:{} == ti:{}".format(pos["id"], ti_old, ti))
                    else:
                        print("[POS] id:{} has no old ti".format(pos["id"]))

                if ti_old and ti != ti_old:
                    output = [
                        pos["name"],
                        "",
                        ti_old,
                        ti
                    ]
                    self.output_q.put(output)
                    self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



# -*- coding: utf-8 -*-
import click
import subprocess
import os
from libcnmc.utils import N_PROC
from libcnmc.core.backend import OOOPFactory
from libcnmc import cir_4_2015
from datetime import datetime
from libcnmc.utils import merge_procs


@click.group()
def cnmc_4_2015():
    pass


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069,
              help='Port servidor ERP', type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def cir_4_2015_f10at(**kwargs):
    """
    Click entry to gnerate the F10AT of 4/2015

    :param kwargs: Params to pass to the process
    :type kwargs: dict(str, str)
    :return: None
    :rtype: None
    """

    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.F10AT(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        codi_r1=kwargs['codi_r1'],
        year=kwargs['year']
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
@click.option('--derechos/--no-derechos', default=False)
@click.option("--reducir-cups/--no-reducir-cups", default=False)
@click.option("--allow-cna/--no-allow-cna", default=False)
@click.option("--zona_qualitat",default="ct")
def cir_4_2015_f1(**kwargs):
    """
    Click entry to generate the F1 of 4/2015

    :param kwargs: Params to pass to the process
    :type kwargs: dict(str,str)
    :return: None
    :rtype: None
    """
    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.F1(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        codi_r1=kwargs['codi_r1'],
        year=kwargs['year'],
        derechos=kwargs["derechos"],
        reducir_cups=kwargs["reducir_cups"],
        allow_cna=kwargs["allow_cna"],
        zona_qualitat=kwargs["zona_qualitat"]
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
@click.option("--reducir-cups/--no-reducir-cups",default=False)
def cir_4_2015_f11(**kwargs):
    """
    Click entry to generate the F11 of 4/2015

    :param kwargs: Params to pass to the process
    :type kwargs: dict(str,str)
    :return: None
    :rtype: None
    """

    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.F11(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        codi_r1=kwargs['codi_r1'],
        year=kwargs['year'],
        reducir_cups=kwargs["reducir_cups"]
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069,
              help='Port servidor ERP', type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
@click.option('--derechos/--no-derechos', default=False)
@click.option("--reducir-cups/--no-reducir-cups",default=False)
def cir_4_2015_f1bis(**kwargs):
    """
    Click entry to gneerate the F1BIS of 4/2015

    :param kwargs: Params to pass to the process
    :type kwargs:dict(str, str)
    :return: None
    :rtype: None
    """

    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.F1bis(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        codi_r1=kwargs['codi_r1'],
        year=kwargs['year'],
        derechos=kwargs['derechos'],
        reducir_cups=kwargs["reducir_cups"]
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069,
              help='Port servidor ERP', type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def cir_4_2015_f12(**kwargs):
    """
    Click entry to generate the F12 of the 4/2015

    :param kwargs: Params to pass to the proces
    :type kwargs: dict(str, str)
    :return: None
    :rtype: None
    """
    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.F12(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        year=kwargs['year']
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069,
              help='Port servidor ERP', type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
@click.option('--fiabilitat/--no-fiabilitat', default=True,
              help="Only include fiabilitat elements")
@click.option('--doslmesp/--no-doselemesp', default=True,
              help="Incloure elements 2l+p")
def cir_4_2015_f12bis(**kwargs):
    """
    Click entry to generate the F12 BIS of the 4/2015

    :param kwargs: Params to pass to the process
    :type kwargs: dict(str, str)
    :return: None
    :rtype: None
    """

    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.F12bis(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        year=kwargs['year'],
        fiabilitat=kwargs["fiabilitat"],
        doslmesp=kwargs["doslmesp"]
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069,
              help='Port servidor ERP', type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def cir_4_2015_f13(**kwargs):
    """
    Click entry to generate the F13 file of 4/2015

    :param kwargs: Params to pass to the process
    :type kwargs: dict(str,str)
    :return: None
    :rtype: None
    """

    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.F13(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        codi_r1=kwargs['codi_r1'],
        year=kwargs['year']
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069,
              help='Port servidor ERP', type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def cir_4_2015_f13bis(**kwargs):
    """
    Click entry to generate the F13 BIS of 4/2015

    :param kwargs: Params to pas to the process
    :type kwargs: dict(str,str)
    :return: None
    :rtype: None
    """

    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.F13bis(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=1,
        year=kwargs['year']
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069,
              help='Port servidor ERP', type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
@click.option('--all-int/--only-int-auto', default=True,
              help="Incloure Pos. amb Int.Auto i Pos. sense Int. o només"
                   "Pos. amb Int.Auto")
def cir_4_2015_f13c(**kwargs):
    """
    Click entry to generate the F13C of 4/2015

    :param kwargs: Params to pass to the process
    :type kwargs: dict(str, str)
    :return: None
    :return: None
    """

    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.F13c(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        year=kwargs['year'],
        all_int=kwargs['all_int']
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069,
              help='Port servidor ERP', type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def cir_4_2015_f14(**kwargs):
    """
    Click entry to generate the F14 file of 4/2015

    :param kwargs: Params to pass to the process
    :type kwargs: dict(str,str)
    :return: None
    :rtype: None
    """

    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.F14(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        year=kwargs['year']
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069,
              help='Port servidor ERP', type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def cir_4_2015_f15(**kwargs):
    """
    Click ent ry to generate F15 of 4/2015

    :param kwargs: Params to pass to the process
    :type kwargs: dict(str, str)
    :return: None
    :rtype: None
    """

    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])

    proc = cir_4_2015.F15(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        codi_r1=kwargs['codi_r1'],
        year=kwargs['year']
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069,
              help='Port servidor ERP', type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def cir_4_2015_f10bt(**kwargs):
    """
    Click entry to genertate the F10BT fo 4/2015

    :param kwargs: Params to pass to the process
    :type kwargs: dict(str,str)
    :return: None
    :rtype: None
    """

    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.F10BT(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        codi_r1=kwargs['codi_r1'],
        year=kwargs['year']
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069,
              help='Port servidor ERP', type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
@click.option('--derechos/--no-derechos', default=False)
@click.option("--reducir-cups/--no-reducir-cups",default=False)
def cir_4_2015_f20(**kwargs):
    """
    Click entry to generate the F20 file of 4/2015

    :param kwargs: Params to pass to the process
    :type kwargs:dict(str,str)
    :return: None
    :rtype: None
    """
    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.F20(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        codi_r1=kwargs['codi_r1'],
        year=kwargs['year'],
        derechos=kwargs['derechos'],
        reducir_cupss=kwargs["reducir_cups"]
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069,
              help='Port servidor ERP', type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
@click.option(
    '--alternative/--no-alternative', default=False,
    help="Generar l'F9 amb una linia per a cada coordenada dels trams"
)
def cir_4_2015_f9(**kwargs):
    """
    Click entry to generate the F9 of 4/20105

    :param kwargs: Params to pass to the process
    :type kwargs: dict(str, str)
    :return: None
    :rtype: None
    """

    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.F9(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        year=kwargs['year'],
        alternative=kwargs['alternative']
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
@click.option('-f', '--file-input', type=click.Path(exists=True))
def cir_4_2015_create_celles(**kwargs):
    """
    Click entry to create celles of 4/2015

    :param kwargs: Params to pass to the process
    :type kwargs: dict(str,str)
    :return: None
    :rtype: None
    """

    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.CreateCelles4_2015(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        connection=O,
        num_proc=kwargs['num_proc'],
        file_input=kwargs['file_input'])
    proc.calc()


@cnmc_4_2015.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--num-proc', default=N_PROC, type=click.INT)
def cir_4_2015_f16(**kwargs):
    """
    Click entry to generate the F16 of 4/2015

    :param kwargs: Params to pass to the process
    :type kwargs: dict(str,str)
    :return: None
    :rtype: None
    """
    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = cir_4_2015.F16(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        codi_r1=kwargs['codi_r1'],
        year=kwargs['year']
    )
    proc.calc()


@cnmc_4_2015.command()
@click.option('-d', '--dir', help='Ruta de la carpeta amb els formularis')
@click.option('-l', '--lang', help='Idioma amb el qual es generaràn les '
                                   'validacions (es_ES o ca_ES)')
def validate_files(**kwargs):
    from libcnmc import checker
    if kwargs['dir'] and kwargs['lang']:
        if os.path.exists(kwargs['dir']):
            if "cli_4_2015.pyc" in __file__:
                path = str(__file__).replace("/cli_4_2015.pyc", "")
            else:
                path = str(__file__).replace("/cli_4_2015.py", "")
            checker_file = '{}/checker.py'.format(path)
            print(subprocess.check_output(
                ['python', checker_file, '--dir={}'.format(kwargs['dir']),
                 '--lang={}'.format(kwargs['lang'])]
            ))


# -*- coding: utf-8 -*-
from datetime import datetime
import traceback

from libcnmc.utils import get_ine, format_f, convert_srid, get_srid
from libcnmc.core import StopMultiprocessBased
from shapely import wkt


class F16(StopMultiprocessBased):
    def __init__(self, **kwargs):
        super(F16, self).__init__(**kwargs)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'F16 - Condensadors'
        self.base_object = 'Condensadors'

    def get_sequence(self):
        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        search_params = ['|', ('data_pm', '=', False),
                         ('data_pm', '<', data_pm),
                         '|', ('data_baixa', '>', data_baixa),
                         ('data_baixa', '=', False)]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataCondensadors.search(
            search_params, 0, 0, False, {'active_test': False}
        )

    def get_node_vertex(self, cond_name):
        """
        Returns the node and vertex of the condensador
        :param cond_name:
        :type cond_name: str
        :return: node,vertex
        :rtype: (str, str)
        """

        O = self.connection
        ident = O.GiscegisElementsbt.search([('codi', '=', cond_name)])
        if not ident:
            ident = O.GiscegisElementsat.search([('codi', '=', cond_name)])
            if ident:
                data = O.GiscegisElementsat.read(ident[0], ["node", "vertex"])
        else:
            data = O.GiscegisElementsbt.read(ident[0], ["node", "vertex"])

        if not ident:
            node = ''
            vertex = ''
        else:
            node = data["node"][1]
            x, y = data["vertex"][1].split(",")
            vertex = (round(float(x), 3), round(float(y), 3))

        return node, vertex

    def get_ine(self, municipi_id):
        O = self.connection
        muni = O.ResMunicipi.read(municipi_id, ['ine', 'dc'])
        return get_ine(O, muni['ine'])

    def get_dades_ct(self, ct_id):
        O = self.connection
        ct = O.GiscedataCts.read(ct_id, ['id_municipi', 'propietari'])
        return ct

    def get_tensio(self, tensio_id):
        O = self.connection
        tensio = O.GiscedataTensionsTensio.read(tensio_id, ['tensio'])
        return tensio['tensio']

    def consumer(self):
        o_codi_r1 = 'R1-%s' % self.codi_r1[-3:]
        O = self.connection
        fields_to_read = [
            'name', 'cini', 'ct_id', 'tensio_id', 'potencia_instalada',
            "node_id"
        ]
        while True:
            try:
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                condensador = O.GiscedataCondensadors.read(item, fields_to_read)
                o_cond = condensador['name']
                o_cini = condensador.get('cini', '')
                o_potencia = condensador['potencia_instalada']

                if "node_id" in condensador:
                    o_node = condensador["node_id"][1]
                    geom = O.GiscegisNodes.read(condensador["node_id"][0], ["geom"])
                    posicion = wkt.loads(geom["geom"]).coords[0]
                else:
                    o_node, posicion = self.get_node_vertex(condensador['name'])

                o_node = o_node.replace('*', '')

                ct = self.get_dades_ct(condensador['ct_id'][0])
                o_propietari = int(ct['propietari'])

                o_ine_muni, o_ine_prov = '', ''
                if ct['id_municipi']:
                    o_ine_prov, o_ine_muni = self.get_ine(ct['id_municipi'][0])
                o_tensio = format_f(
                    float(self.get_tensio(condensador['tensio_id'][0])) / 1000.0,
                    decimals=3
                )

                o_any = self.year
                z = ''
                res_srid = ['', '']
                if posicion:
                    res_srid = convert_srid(get_srid(O), posicion)
                self.output_q.put([
                    o_node,                             # NUDO
                    o_cond,                             # CONDENSADOR
                    o_cini,                             # CINI
                    format_f(res_srid[0], decimals=3),  # X
                    format_f(res_srid[1], decimals=3),  # Y
                    z,                                  # Z
                    o_ine_muni,                         # MUNICIPIO
                    o_ine_prov,                         # PROVINCIA
                    o_tensio,                           # NIVEL TENSION
                    format_f(o_potencia, decimals=3),   # POTENCIA INSTALADA
                    o_codi_r1,                          # CODIGO DISTRIBUIDORA
                    o_propietari,                       # PROPIETARIO
                    o_any                               # AÑO INFORMACION
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



# -*- coding: utf-8 -*-
import traceback
from datetime import datetime

from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import get_ine, format_f


class FD1(StopMultiprocessBased):
    def __init__(self, **kwargs):
        super(FD1, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.report_name = "Formulario D1: Continuidad del suministro"
        self.base_object = 'D1'

    def get_sequence(self):
        prevision_ids = self.connection.model('cir8.2021.d1').search(
            [('year', '=', self.year)]
        )
        return prevision_ids

    def consumer(self):
        O = self.connection
        fields_to_read = []
        while True:
            try:
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                d1 = O.model('cir8.2021.d1').read(item, fields_to_read)
                o_ccaa = O.ResComunitat_autonoma.get_ccaa_from_municipi(
                    d1['municipio'][0]
                )[0]
                ine = O.ResMunicipi.read(d1['municipio'][0], ['ine'])['ine']
                o_provincia, o_municipio = get_ine(O, ine)
                self.output_q.put([
                    d1['tipo'],
                    o_municipio,
                    o_provincia,
                    o_ccaa,
                    d1['zona'],
                    format_f(d1['potencia'], 3),
                    d1['n_cups'],
                    format_f(d1['programadas_transporte'], 2),
                    format_f(d1['programadas_distribucion'], 2),
                    format_f(d1['programadas_total'], 2),
                    format_f(d1['imprevistas_generacion'], 2),
                    format_f(d1['imprevistas_transporte'], 2),
                    format_f(d1['imprevistas_terceros'], 2),
                    format_f(d1['imprevistas_fuerza_mayor'], 2),
                    format_f(d1['imprevistas_propias'], 2),
                    format_f(d1['total'], 2)
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
